@@ -24,7 +24,7 @@ void bpf_try_compile(const char *rulefile, struct sock_fprog *bpf, uint32_t link
 		panic("Cannot compile filter %s\n", rulefile);
 
 	bpf->len = _bpf.bf_len;
-	bpf->filter = xrealloc(bpf->filter, 1, bpf->len * sizeof(*out));
+	bpf->filter = xrealloc(bpf->filter, bpf->len * sizeof(*out));
 
 	for (i = 0, ins = _bpf.bf_insns, out = bpf->filter; i < bpf->len;
 	     ++i, ++ins, ++out) {
