@@ -17,6 +17,7 @@
 #include "ring.h"
 #include "tprintf.h"
 #include "linktype.h"
+#include "proto_vlan.h"
 
 #define PRINT_NORM		0
 #define PRINT_LESS		1
@@ -33,8 +34,8 @@ static const char * const packet_types[256] = {
 	[PACKET_MULTICAST]	=	"M",  /* Multicast */
 	[PACKET_OTHERHOST]	=	"P",  /* Promisc */
 	[PACKET_OUTGOING]	=	">",  /* Outgoing */
-	[PACKET_USER]		=	">U", /* To Userspace */
-	[PACKET_KERNEL]		=	">K", /* To Kernelspace */
+	[PACKET_USER]		=	"K->U", /* To Userspace */
+	[PACKET_KERNEL]		=	"U->K", /* To Kernelspace */
 };
 
 static inline const char *__show_ts_source(uint32_t status)
@@ -91,6 +92,17 @@ static inline void __show_frame_hdr(uint8_t *packet, size_t len, int linktype,
 			tpacket_uhdr(hdr, tp_nsec, v3),
 			count,
 			v3 ? "" : __show_ts_source(hdr.h2->tp_status));
+
+		if (tpacket_has_vlan_info(&hdr)) {
+			uint16_t tci = tpacket_uhdr_vlan_tci(&hdr, v3);
+
+			tprintf(" [ tpacketv3 VLAN ");
+			tprintf("Prio (%u), ", vlan_tci2prio(tci));
+			tprintf("CFI (%u), ", vlan_tci2cfi(tci));
+			tprintf("ID (%u), ", vlan_tci2vid(tci));
+			tprintf("Proto (0x%.4x)", tpacket_uhdr_vlan_proto(&hdr, v3));
+			tprintf(" ]\n");
+		}
 		break;
 	}
 }
@@ -105,7 +117,7 @@ static inline void show_frame_hdr(uint8_t *packet, size_t len, int linktype,
 
 extern void dissector_init_all(int fnttype);
 extern void dissector_entry_point(uint8_t *packet, size_t len, int linktype,
-				  int mode, uint16_t proto);
+				  int mode, struct sockaddr_ll *sll);
 extern void dissector_cleanup_all(void);
 extern int dissector_set_print_type(void *ptr, int type);
 
