@@ -19,10 +19,14 @@
 #include <curses.h>
 #include <dirent.h>
 #include <sys/stat.h>
+#include <sys/time.h>
 #include <sys/fsuid.h>
 #include <urcu.h>
 #include <libgen.h>
 #include <inttypes.h>
+#include <poll.h>
+#include <fcntl.h>
+#include <arpa/inet.h>
 
 #include "die.h"
 #include "xmalloc.h"
@@ -36,6 +40,16 @@
 #include "locking.h"
 #include "pkt_buff.h"
 #include "screen.h"
+#include "proc.h"
+#include "sysctl.h"
+
+#ifndef NSEC_PER_SEC
+#define NSEC_PER_SEC 1000000000L
+#endif
+
+#ifndef USEC_PER_SEC
+#define USEC_PER_SEC 1000000L
+#endif
 
 struct flow_entry {
 	uint32_t flow_id, use, status;
@@ -44,15 +58,23 @@ struct flow_entry {
 	uint32_t ip6_src_addr[4], ip6_dst_addr[4];
 	uint16_t port_src, port_dst;
 	uint8_t  tcp_state, tcp_flags, sctp_state, dccp_state;
-	uint64_t counter_pkts, counter_bytes;
+	uint64_t pkts_src, bytes_src;
+	uint64_t pkts_dst, bytes_dst;
 	uint64_t timestamp_start, timestamp_stop;
 	char country_src[128], country_dst[128];
 	char city_src[128], city_dst[128];
 	char rev_dns_src[256], rev_dns_dst[256];
-	char cmdline[256];
+	char procname[256];
 	struct flow_entry *next;
 	int inode;
 	unsigned int procnum;
+	bool is_visible;
+	struct nf_conntrack *ct;
+	struct timeval last_update;
+	double rate_bytes_src;
+	double rate_bytes_dst;
+	double rate_pkts_src;
+	double rate_pkts_dst;
 };
 
 struct flow_list {
@@ -60,6 +82,11 @@ struct flow_list {
 	struct spinlock lock;
 };
 
+enum flow_direction {
+	FLOW_DIR_SRC,
+	FLOW_DIR_DST,
+};
+
 #ifndef ATTR_TIMESTAMP_START
 # define ATTR_TIMESTAMP_START 63
 #endif
@@ -77,12 +104,29 @@ struct flow_list {
 #define INCLUDE_ICMP	(1 << 5)
 #define INCLUDE_SCTP	(1 << 6)
 
-static volatile sig_atomic_t sigint = 0;
-static int what = INCLUDE_IPV4 | INCLUDE_IPV6 | INCLUDE_TCP, show_src = 0;
-static struct flow_list flow_list;
-static struct condlock collector_ready;
+struct sysctl_params_ctx {
+	int nfct_acct;
+	int nfct_tstamp;
+};
 
-static const char *short_options = "vhTUsDIS46u";
+enum rate_units {
+	RATE_BITS,
+	RATE_BYTES
+};
+
+static volatile bool is_flow_collecting;
+static volatile sig_atomic_t sigint = 0;
+static int what = INCLUDE_IPV4 | INCLUDE_IPV6 | INCLUDE_TCP;
+static struct flow_list flow_list;
+static struct sysctl_params_ctx sysctl = { -1, -1 };
+
+static unsigned int interval = 1;
+static bool show_src = false;
+static bool resolve_dns = true;
+static bool resolve_geoip = true;
+static enum rate_units rate_type = RATE_BYTES;
+
+static const char *short_options = "vhTUsDIS46ut:nGb";
 static const struct option long_options[] = {
 	{"ipv4",	no_argument,		NULL, '4'},
 	{"ipv6",	no_argument,		NULL, '6'},
@@ -91,14 +135,18 @@ static const struct option long_options[] = {
 	{"dccp",	no_argument,		NULL, 'D'},
 	{"icmp",	no_argument,		NULL, 'I'},
 	{"sctp",	no_argument,		NULL, 'S'},
+	{"no-dns",      no_argument,		NULL, 'n'},
+	{"no-geoip",    no_argument,		NULL, 'G'},
 	{"show-src",	no_argument,		NULL, 's'},
+	{"bits",        no_argument,		NULL, 'b'},
 	{"update",	no_argument,		NULL, 'u'},
+	{"interval",    required_argument,	NULL, 't'},
 	{"version",	no_argument,		NULL, 'v'},
 	{"help",	no_argument,		NULL, 'h'},
 	{NULL, 0, NULL, 0}
 };
 
-static const char *copyright = "Please report bugs to <bugs@netsniff-ng.org>\n"
+static const char *copyright = "Please report bugs to <netsniff-ng@googlegroups.com>\n"
 	"Copyright (C) 2011-2013 Daniel Borkmann <dborkma@tik.ee.ethz.ch>\n"
 	"Copyright (C) 2011-2012 Emmanuel Roullit <emmanuel.roullit@gmail.com>\n"
 	"Swiss federal institute of technology (ETH Zurich)\n"
@@ -146,19 +194,6 @@ static const char *const tcp_state2str[TCP_CONNTRACK_MAX] = {
 	[TCP_CONNTRACK_SYN_SENT2]	= "SYN_SENT2",
 };
 
-static const uint8_t tcp_states[] = {
-	TCP_CONNTRACK_SYN_SENT,
-	TCP_CONNTRACK_SYN_RECV,
-	TCP_CONNTRACK_ESTABLISHED,
-	TCP_CONNTRACK_FIN_WAIT,
-	TCP_CONNTRACK_CLOSE_WAIT,
-	TCP_CONNTRACK_LAST_ACK,
-	TCP_CONNTRACK_TIME_WAIT,
-	TCP_CONNTRACK_CLOSE,
-	TCP_CONNTRACK_SYN_SENT2,
-	TCP_CONNTRACK_NONE,
-};
-
 static const char *const dccp_state2str[DCCP_CONNTRACK_MAX] = {
 	[DCCP_CONNTRACK_NONE]		= "NOSTATE",
 	[DCCP_CONNTRACK_REQUEST]	= "REQUEST",
@@ -172,19 +207,6 @@ static const char *const dccp_state2str[DCCP_CONNTRACK_MAX] = {
 	[DCCP_CONNTRACK_INVALID]	= "INVALID",
 };
 
-static const uint8_t dccp_states[] = {
-	DCCP_CONNTRACK_NONE,
-	DCCP_CONNTRACK_REQUEST,
-	DCCP_CONNTRACK_RESPOND,
-	DCCP_CONNTRACK_PARTOPEN,
-	DCCP_CONNTRACK_OPEN,
-	DCCP_CONNTRACK_CLOSEREQ,
-	DCCP_CONNTRACK_CLOSING,
-	DCCP_CONNTRACK_TIMEWAIT,
-	DCCP_CONNTRACK_IGNORE,
-	DCCP_CONNTRACK_INVALID,
-};
-
 static const char *const sctp_state2str[SCTP_CONNTRACK_MAX] = {
 	[SCTP_CONNTRACK_NONE]		= "NOSTATE",
 	[SCTP_CONNTRACK_CLOSED]		= "CLOSED",
@@ -196,17 +218,6 @@ static const char *const sctp_state2str[SCTP_CONNTRACK_MAX] = {
 	[SCTP_CONNTRACK_SHUTDOWN_ACK_SENT] = "SHUTDOWN_ACK_SENT",
 };
 
-static const uint8_t sctp_states[] = {
-	SCTP_CONNTRACK_NONE,
-	SCTP_CONNTRACK_CLOSED,
-	SCTP_CONNTRACK_COOKIE_WAIT,
-	SCTP_CONNTRACK_COOKIE_ECHOED,
-	SCTP_CONNTRACK_ESTABLISHED,
-	SCTP_CONNTRACK_SHUTDOWN_SENT,
-	SCTP_CONNTRACK_SHUTDOWN_RECD,
-	SCTP_CONNTRACK_SHUTDOWN_ACK_SENT,
-};
-
 static const struct nfct_filter_ipv4 filter_ipv4 = {
 	.addr = __constant_htonl(INADDR_LOOPBACK),
 	.mask = 0xffffffff,
@@ -217,6 +228,18 @@ static const struct nfct_filter_ipv6 filter_ipv6 = {
 	.mask = { 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff },
 };
 
+static int64_t time_after_us(struct timeval *tv)
+{
+	struct timeval now;
+
+	bug_on(gettimeofday(&now, NULL));
+
+	now.tv_sec  -= tv->tv_sec;
+	now.tv_usec -= tv->tv_usec;
+
+	return now.tv_sec * USEC_PER_SEC + now.tv_usec;
+}
+
 static void signal_handler(int number)
 {
 	switch (number) {
@@ -231,7 +254,7 @@ static void signal_handler(int number)
 	}
 }
 
-static void flow_entry_from_ct(struct flow_entry *n, struct nf_conntrack *ct);
+static void flow_entry_from_ct(struct flow_entry *n, const struct nf_conntrack *ct);
 static void flow_entry_get_extended(struct flow_entry *n);
 
 static void help(void)
@@ -248,8 +271,12 @@ static void help(void)
 	     "  -D|--dccp              Show only DCCP flows\n"
 	     "  -I|--icmp              Show only ICMP/ICMPv6 flows\n"
 	     "  -S|--sctp              Show only SCTP flows\n"
+	     "  -n|--no-dns            Don't perform hostname lookup\n"
+	     "  -G|--no-geoip          Don't perform GeoIP lookup\n"
 	     "  -s|--show-src          Also show source, not only dest\n"
+	     "  -b|--bits              Show rates in bits/s instead of bytes/s\n"
 	     "  -u|--update            Update GeoIP databases\n"
+	     "  -t|--interval <time>   Refresh time in seconds (default 1s)\n"
 	     "  -v|--version           Print version and exit\n"
 	     "  -h|--help              Print this help and exit\n\n"
 	     "Examples:\n"
@@ -272,6 +299,32 @@ static void version(void)
 	die();
 }
 
+static void flow_entry_update_time(struct flow_entry *n)
+{
+	bug_on(gettimeofday(&n->last_update, NULL));
+}
+
+#define CALC_RATE(fld) do {							\
+	n->rate_##fld = (((fld) > n->fld) ? (((fld) - n->fld) / sec) : 0);	\
+} while (0)
+
+static void flow_entry_calc_rate(struct flow_entry *n, const struct nf_conntrack *ct)
+{
+	uint64_t bytes_src = nfct_get_attr_u64(ct, ATTR_ORIG_COUNTER_BYTES);
+	uint64_t bytes_dst = nfct_get_attr_u64(ct, ATTR_REPL_COUNTER_BYTES);
+	uint64_t pkts_src  = nfct_get_attr_u64(ct, ATTR_ORIG_COUNTER_PACKETS);
+	uint64_t pkts_dst  = nfct_get_attr_u64(ct, ATTR_REPL_COUNTER_PACKETS);
+	double sec = (double)time_after_us(&n->last_update) / USEC_PER_SEC;
+
+	if (sec < 1)
+		return;
+
+	CALC_RATE(bytes_src);
+	CALC_RATE(bytes_dst);
+	CALC_RATE(pkts_src);
+	CALC_RATE(pkts_dst);
+}
+
 static inline struct flow_entry *flow_entry_xalloc(void)
 {
 	return xzmalloc(sizeof(struct flow_entry));
@@ -279,6 +332,9 @@ static inline struct flow_entry *flow_entry_xalloc(void)
 
 static inline void flow_entry_xfree(struct flow_entry *n)
 {
+	if (n->ct)
+		nfct_destroy(n->ct);
+
 	xfree(n);
 }
 
@@ -288,10 +344,29 @@ static inline void flow_list_init(struct flow_list *fl)
 	spinlock_init(&fl->lock);
 }
 
-static void flow_list_new_entry(struct flow_list *fl, struct nf_conntrack *ct)
+static inline bool nfct_is_dns(const struct nf_conntrack *ct)
 {
-	struct flow_entry *n = flow_entry_xalloc();
+	uint16_t port_src = nfct_get_attr_u16(ct, ATTR_ORIG_PORT_SRC);
+	uint16_t port_dst = nfct_get_attr_u16(ct, ATTR_ORIG_PORT_DST);
 
+	return ntohs(port_src) == 53 || ntohs(port_dst) == 53;
+}
+
+static void flow_list_new_entry(struct flow_list *fl, const struct nf_conntrack *ct)
+{
+	struct flow_entry *n;
+
+	/* We don't want to analyze / display DNS itself, since we
+	 * use it to resolve reverse dns.
+	 */
+	if (nfct_is_dns(ct))
+		return;
+
+	n = flow_entry_xalloc();
+
+	n->ct = nfct_clone(ct);
+
+	flow_entry_update_time(n);
 	flow_entry_from_ct(n, ct);
 	flow_entry_get_extended(n);
 
@@ -314,47 +389,40 @@ static struct flow_entry *flow_list_find_id(struct flow_list *fl,
 	return NULL;
 }
 
-static struct flow_entry *flow_list_find_prev_id(struct flow_list *fl,
+static struct flow_entry *flow_list_find_prev_id(const struct flow_list *fl,
 						 uint32_t id)
 {
-	struct flow_entry *n = rcu_dereference(fl->head), *tmp;
+	struct flow_entry *prev = rcu_dereference(fl->head), *next;
 
-	if (n->flow_id == id)
+	if (prev->flow_id == id)
 		return NULL;
 
-	while ((tmp = rcu_dereference(n->next)) != NULL) {
-		if (tmp->flow_id == id)
-			return n;
+	while ((next = rcu_dereference(prev->next)) != NULL) {
+		if (next->flow_id == id)
+			return prev;
 
-		n = tmp;
+		prev = next;
 	}
 
 	return NULL;
 }
 
 static void flow_list_update_entry(struct flow_list *fl,
-				   struct nf_conntrack *ct)
+				   const struct nf_conntrack *ct)
 {
-	int do_ext = 0;
 	struct flow_entry *n;
 
 	n = flow_list_find_id(fl, nfct_get_attr_u32(ct, ATTR_ID));
 	if (n == NULL) {
-		n = flow_entry_xalloc();
-		do_ext = 1;
+		flow_list_new_entry(fl, ct);
+		return;
 	}
 
 	flow_entry_from_ct(n, ct);
-	if (do_ext) {
-		flow_entry_get_extended(n);
-
-		rcu_assign_pointer(n->next, fl->head);
-		rcu_assign_pointer(fl->head, n);
-	}
 }
 
 static void flow_list_destroy_entry(struct flow_list *fl,
-				    struct nf_conntrack *ct)
+				    const struct nf_conntrack *ct)
 {
 	struct flow_entry *n1, *n2;
 	uint32_t id = nfct_get_attr_u32(ct, ATTR_ID);
@@ -368,8 +436,10 @@ static void flow_list_destroy_entry(struct flow_list *fl,
 
 			flow_entry_xfree(n1);
 		} else {
+			struct flow_entry *next = fl->head->next;
+
 			flow_entry_xfree(fl->head);
-			fl->head = NULL;
+			fl->head = next;
 		}
 	}
 }
@@ -415,15 +485,14 @@ static int walk_process(unsigned int pid, struct flow_entry *n)
 			continue;
 
 		if (S_ISSOCK(statbuf.st_mode) && (ino_t) n->inode == statbuf.st_ino) {
-			memset(n->cmdline, 0, sizeof(n->cmdline));
+			char cmdline[256];
 
-			snprintf(path, sizeof(path), "/proc/%u/exe", pid);
-
-			ret = readlink(path, n->cmdline,
-				       sizeof(n->cmdline) - 1);
+			ret = proc_get_cmdline(pid, cmdline, sizeof(cmdline));
 			if (ret < 0)
-				panic("readlink error: %s\n", strerror(errno));
+				panic("Failed to get process cmdline: %s\n", strerror(errno));
 
+			if (snprintf(n->procname, sizeof(n->procname), "%s", basename(cmdline)) < 0)
+				n->procname[0] = '\0';
 			n->procnum = pid;
 			closedir(dir);
 			return 1;
@@ -442,7 +511,7 @@ static void walk_processes(struct flow_entry *n)
 
 	/* n->inode must be set */
 	if (n->inode <= 0) {
-		memset(n->cmdline, 0, sizeof(n->cmdline));
+		n->procname[0] = '\0';
 		return;
 	}
 
@@ -467,7 +536,7 @@ static void walk_processes(struct flow_entry *n)
 	closedir(dir);
 }
 
-static int get_port_inode(uint16_t port, int proto, int is_ip6)
+static int get_port_inode(uint16_t port, int proto, bool is_ip6)
 {
 	int ret = -ENOENT;
 	char path[128], buff[1024];
@@ -511,7 +580,7 @@ static int get_port_inode(uint16_t port, int proto, int is_ip6)
 		memcpy(n->elem, buff, sizeof(n->elem));	\
 } while (0)
 
-static void flow_entry_from_ct(struct flow_entry *n, struct nf_conntrack *ct)
+static void flow_entry_from_ct(struct flow_entry *n, const struct nf_conntrack *ct)
 {
 	CP_NFCT(l3_proto, ATTR_ORIG_L3PROTO, 8);
 	CP_NFCT(l4_proto, ATTR_ORIG_L4PROTO, 8);
@@ -529,8 +598,11 @@ static void flow_entry_from_ct(struct flow_entry *n, struct nf_conntrack *ct)
 	CP_NFCT(sctp_state, ATTR_SCTP_STATE, 8);
 	CP_NFCT(dccp_state, ATTR_DCCP_STATE, 8);
 
-	CP_NFCT(counter_pkts, ATTR_ORIG_COUNTER_PACKETS, 64);
-	CP_NFCT(counter_bytes, ATTR_ORIG_COUNTER_BYTES, 64);
+	CP_NFCT(pkts_src, ATTR_ORIG_COUNTER_PACKETS, 64);
+	CP_NFCT(bytes_src, ATTR_ORIG_COUNTER_BYTES, 64);
+
+	CP_NFCT(pkts_dst, ATTR_REPL_COUNTER_PACKETS, 64);
+	CP_NFCT(bytes_dst, ATTR_REPL_COUNTER_BYTES, 64);
 
 	CP_NFCT(timestamp_start, ATTR_TIMESTAMP_START, 64);
 	CP_NFCT(timestamp_stop, ATTR_TIMESTAMP_STOP, 64);
@@ -548,24 +620,11 @@ static void flow_entry_from_ct(struct flow_entry *n, struct nf_conntrack *ct)
 	n->ip4_dst_addr = ntohl(n->ip4_dst_addr);
 }
 
-enum flow_entry_direction {
-	flow_entry_src,
-	flow_entry_dst,
-};
-
-static inline int flow_entry_get_extended_is_dns(struct flow_entry *n)
-{
-	/* We don't want to analyze / display DNS itself, since we
-	 * use it to resolve reverse dns.
-	 */
-	return n->port_src == 53 || n->port_dst == 53;
-}
-
 #define SELFLD(dir,src_member,dst_member)	\
-	(((dir) == flow_entry_src) ? n->src_member : n->dst_member)
+	(((dir) == FLOW_DIR_SRC) ? n->src_member : n->dst_member)
 
-static void flow_entry_get_sain4_obj(struct flow_entry *n,
-				     enum flow_entry_direction dir,
+static void flow_entry_get_sain4_obj(const struct flow_entry *n,
+				     enum flow_direction dir,
 				     struct sockaddr_in *sa)
 {
 	memset(sa, 0, sizeof(*sa));
@@ -573,8 +632,8 @@ static void flow_entry_get_sain4_obj(struct flow_entry *n,
 	sa->sin_addr.s_addr = htonl(SELFLD(dir, ip4_src_addr, ip4_dst_addr));
 }
 
-static void flow_entry_get_sain6_obj(struct flow_entry *n,
-				     enum flow_entry_direction dir,
+static void flow_entry_get_sain6_obj(const struct flow_entry *n,
+				     enum flow_direction dir,
 				     struct sockaddr_in6 *sa)
 {
 	memset(sa, 0, sizeof(*sa));
@@ -586,7 +645,7 @@ static void flow_entry_get_sain6_obj(struct flow_entry *n,
 
 static void
 flow_entry_geo_city_lookup_generic(struct flow_entry *n,
-				   enum flow_entry_direction dir)
+				   enum flow_direction dir)
 {
 	struct sockaddr_in sa4;
 	struct sockaddr_in6 sa6;
@@ -607,20 +666,18 @@ flow_entry_geo_city_lookup_generic(struct flow_entry *n,
 		break;
 	}
 
-	bug_on(sizeof(n->city_src) != sizeof(n->city_dst));
+	build_bug_on(sizeof(n->city_src) != sizeof(n->city_dst));
 
-	if (city) {
-		memcpy(SELFLD(dir, city_src, city_dst), city,
-		       min(sizeof(n->city_src), strlen(city)));
-	} else {
-		memset(SELFLD(dir, city_src, city_dst), 0,
-		       sizeof(n->city_src));
-	}
+	if (city)
+		strlcpy(SELFLD(dir, city_src, city_dst), city,
+		        sizeof(n->city_src));
+	else
+		SELFLD(dir, city_src, city_dst)[0] = '\0';
 }
 
 static void
 flow_entry_geo_country_lookup_generic(struct flow_entry *n,
-				      enum flow_entry_direction dir)
+				      enum flow_direction dir)
 {
 	struct sockaddr_in sa4;
 	struct sockaddr_in6 sa6;
@@ -641,26 +698,26 @@ flow_entry_geo_country_lookup_generic(struct flow_entry *n,
 		break;
 	}
 
-	bug_on(sizeof(n->country_src) != sizeof(n->country_dst));
+	build_bug_on(sizeof(n->country_src) != sizeof(n->country_dst));
 
-	if (country) {
-		memcpy(SELFLD(dir, country_src, country_dst), country,
-		       min(sizeof(n->country_src), strlen(country)));
-	} else {
-		memset(SELFLD(dir, country_src, country_dst), 0,
-		       sizeof(n->country_src));
-	}
+	if (country)
+		strlcpy(SELFLD(dir, country_src, country_dst), country,
+		        sizeof(n->country_src));
+	else
+		SELFLD(dir, country_src, country_dst)[0] = '\0';
 }
 
 static void flow_entry_get_extended_geo(struct flow_entry *n,
-					enum flow_entry_direction dir)
+					enum flow_direction dir)
 {
-	flow_entry_geo_city_lookup_generic(n, dir);
-	flow_entry_geo_country_lookup_generic(n, dir);
+	if (resolve_geoip) {
+		flow_entry_geo_city_lookup_generic(n, dir);
+		flow_entry_geo_country_lookup_generic(n, dir);
+	}
 }
 
 static void flow_entry_get_extended_revdns(struct flow_entry *n,
-					   enum flow_entry_direction dir)
+					   enum flow_direction dir)
 {
 	size_t sa_len;
 	struct sockaddr_in sa4;
@@ -668,12 +725,22 @@ static void flow_entry_get_extended_revdns(struct flow_entry *n,
 	struct sockaddr *sa;
 	struct hostent *hent;
 
+	build_bug_on(sizeof(n->rev_dns_src) != sizeof(n->rev_dns_dst));
+
 	switch (n->l3_proto) {
 	default:
 		bug();
 
 	case AF_INET:
 		flow_entry_get_sain4_obj(n, dir, &sa4);
+
+		if (!resolve_dns) {
+			inet_ntop(AF_INET, &sa4.sin_addr,
+				  SELFLD(dir, rev_dns_src, rev_dns_dst),
+				  sizeof(n->rev_dns_src));
+			return;
+		}
+
 		sa = (struct sockaddr *) &sa4;
 		sa_len = sizeof(sa4);
 		hent = gethostbyaddr(&sa4.sin_addr, sizeof(sa4.sin_addr), AF_INET);
@@ -681,34 +748,40 @@ static void flow_entry_get_extended_revdns(struct flow_entry *n,
 
 	case AF_INET6:
 		flow_entry_get_sain6_obj(n, dir, &sa6);
+
+		if (!resolve_dns) {
+			inet_ntop(AF_INET6, &sa6.sin6_addr,
+				  SELFLD(dir, rev_dns_src, rev_dns_dst),
+				  sizeof(n->rev_dns_src));
+			return;
+		}
+
 		sa = (struct sockaddr *) &sa6;
 		sa_len = sizeof(sa6);
 		hent = gethostbyaddr(&sa6.sin6_addr, sizeof(sa6.sin6_addr), AF_INET6);
 		break;
 	}
 
-	bug_on(sizeof(n->rev_dns_src) != sizeof(n->rev_dns_dst));
 	getnameinfo(sa, sa_len, SELFLD(dir, rev_dns_src, rev_dns_dst),
 		    sizeof(n->rev_dns_src), NULL, 0, NI_NUMERICHOST);
 
-	if (hent) {
-		memset(n->rev_dns_dst, 0, sizeof(n->rev_dns_dst));
-		memcpy(SELFLD(dir, rev_dns_src, rev_dns_dst),
-		       hent->h_name, min(sizeof(n->rev_dns_src),
-					 strlen(hent->h_name)));
-	}
+	if (hent)
+		strlcpy(SELFLD(dir, rev_dns_src, rev_dns_dst), hent->h_name,
+			sizeof(n->rev_dns_src));
 }
 
 static void flow_entry_get_extended(struct flow_entry *n)
 {
-	if (n->flow_id == 0 || flow_entry_get_extended_is_dns(n))
+	if (n->flow_id == 0)
 		return;
 
-	flow_entry_get_extended_revdns(n, flow_entry_src);
-	flow_entry_get_extended_geo(n, flow_entry_src);
+	if (show_src) {
+		flow_entry_get_extended_revdns(n, FLOW_DIR_SRC);
+		flow_entry_get_extended_geo(n, FLOW_DIR_SRC);
+	}
 
-	flow_entry_get_extended_revdns(n, flow_entry_dst);
-	flow_entry_get_extended_geo(n, flow_entry_dst);
+	flow_entry_get_extended_revdns(n, FLOW_DIR_DST);
+	flow_entry_get_extended_geo(n, FLOW_DIR_DST);
 
 	/* Lookup application */
 	n->inode = get_port_inode(n->port_src, n->l4_proto,
@@ -717,7 +790,7 @@ static void flow_entry_get_extended(struct flow_entry *n)
 		walk_processes(n);
 }
 
-static uint16_t presenter_get_port(uint16_t src, uint16_t dst, int tcp)
+static uint16_t presenter_get_port(uint16_t src, uint16_t dst, bool is_tcp)
 {
 	if (src < dst && src < 1024) {
 		return src;
@@ -725,7 +798,7 @@ static uint16_t presenter_get_port(uint16_t src, uint16_t dst, int tcp)
 		return dst;
 	} else {
 		const char *tmp1, *tmp2;
-		if (tcp) {
+		if (is_tcp) {
 			tmp1 = lookup_port_tcp(src);
 			tmp2 = lookup_port_tcp(dst);
 		} else {
@@ -745,7 +818,95 @@ static uint16_t presenter_get_port(uint16_t src, uint16_t dst, int tcp)
 	}
 }
 
-static void presenter_screen_do_line(WINDOW *screen, struct flow_entry *n,
+static char *bandw2str(double bytes, char *buf, size_t len)
+{
+	if (bytes > 1000000000.)
+		snprintf(buf, len, "%.1fGB", bytes / 1000000000.);
+	else if (bytes > 1000000.)
+		snprintf(buf, len, "%.1fMB", bytes / 1000000.);
+	else if (bytes > 1000.)
+		snprintf(buf, len, "%.1fkB", bytes / 1000.);
+	else
+		snprintf(buf, len, "%g bytes", bytes);
+
+	return buf;
+}
+
+static char *rate2str(double rate, char *buf, size_t len)
+{
+	const char * const unit_fmt[2][4] = {
+		{ "%.1fGbit/s", "%.1fMbit/s", "%.1fkbit/s", "%gbit/s" },
+		{ "%.1fGB/s",   "%.1fMB/s",   "%.1fkB/s",   "%gB/s"   }
+	};
+
+	if (rate_type == RATE_BITS)
+		rate *= 8;
+
+	if (rate > 1000000000.)
+		snprintf(buf, len, unit_fmt[rate_type][0], rate / 1000000000.);
+	else if (rate > 1000000.)
+		snprintf(buf, len, unit_fmt[rate_type][1], rate / 1000000.);
+	else if (rate > 1000.)
+		snprintf(buf, len, unit_fmt[rate_type][2], rate / 1000.);
+	else
+		snprintf(buf, len, unit_fmt[rate_type][3], rate);
+
+	return buf;
+}
+
+static void presenter_print_counters(uint64_t bytes, uint64_t pkts,
+				     double rate_bytes, double rate_pkts,
+				     int color)
+{
+	char bytes_str[64];
+
+	printw(" -> (");
+	attron(COLOR_PAIR(color));
+	printw("%"PRIu64" pkts", pkts);
+	if (rate_pkts) {
+		attron(COLOR_PAIR(3));
+		printw("(%.1fpps)", rate_pkts);
+		attron(COLOR_PAIR(color));
+	}
+
+	printw(", %s", bandw2str(bytes, bytes_str, sizeof(bytes_str) - 1));
+	if (rate_bytes) {
+		attron(COLOR_PAIR(3));
+		printw("(%s)", rate2str(rate_bytes, bytes_str,
+			sizeof(bytes_str) - 1));
+		attron(COLOR_PAIR(color));
+	}
+	attroff(COLOR_PAIR(color));
+	printw(")");
+}
+
+static void presenter_print_flow_entry_time(const struct flow_entry *n)
+{
+	int h, m, s;
+	time_t now;
+
+	time(&now);
+
+	s = now - (n->timestamp_start / NSEC_PER_SEC);
+	if (s <= 0)
+		return;
+
+	h = s / 3600;
+	s -= h * 3600;
+	m = s / 60;
+	s -= m * 60;
+
+	printw(" [ time");
+	if (h > 0)
+		printw(" %dh", h);
+	if (m > 0)
+		printw(" %dm", m);
+	if (s > 0)
+		printw(" %ds", s);
+	printw(" ]");
+}
+
+static void presenter_screen_do_line(WINDOW *screen, const struct flow_entry *n,
 				     unsigned int *line)
 {
 	char tmp[128], *pname = NULL;
@@ -755,8 +916,7 @@ static void presenter_screen_do_line(WINDOW *screen, struct flow_entry *n,
 
 	/* PID, application name */
 	if (n->procnum > 0) {
-		slprintf(tmp, sizeof(tmp), "%s(%d)", basename(n->cmdline),
-			 n->procnum);
+		slprintf(tmp, sizeof(tmp), "%s(%d)", n->procname, n->procnum);
 
 		printw("[");
 		attron(COLOR_PAIR(3));
@@ -792,12 +952,12 @@ static void presenter_screen_do_line(WINDOW *screen, struct flow_entry *n,
 	/* Guess application port */
 	switch (n->l4_proto) {
 	case IPPROTO_TCP:
-		port = presenter_get_port(n->port_src, n->port_dst, 1);
+		port = presenter_get_port(n->port_src, n->port_dst, true);
 		pname = lookup_port_tcp(port);
 		break;
 	case IPPROTO_UDP:
 	case IPPROTO_UDPLITE:
-		port = presenter_get_port(n->port_src, n->port_dst, 0);
+		port = presenter_get_port(n->port_src, n->port_dst, false);
 		pname = lookup_port_udp(port);
 		break;
 	}
@@ -806,14 +966,11 @@ static void presenter_screen_do_line(WINDOW *screen, struct flow_entry *n,
 		printw(":%s", pname);
 		attroff(A_BOLD);
 	}
-	printw(" ->");
 
-	/* Number packets, bytes */
-	if (n->counter_pkts > 0 && n->counter_bytes > 0)
-		printw(" (%"PRIu64" pkts, %"PRIu64" bytes) ->",
-		       n->counter_pkts, n->counter_bytes);
+	if (n->timestamp_start > 0)
+		presenter_print_flow_entry_time(n);
 
-	/* Show source information: reverse DNS, port, country, city */
+	/* Show source information: reverse DNS, port, country, city, counters */
 	if (show_src) {
 		attron(COLOR_PAIR(1));
 		mvwprintw(screen, ++(*line), 8, "src: %s", n->rev_dns_src);
@@ -834,10 +991,15 @@ static void presenter_screen_do_line(WINDOW *screen, struct flow_entry *n,
 			printw(")");
 		}
 
+		if (n->pkts_src > 0 && n->bytes_src > 0)
+			presenter_print_counters(n->bytes_src, n->pkts_src,
+						 n->rate_bytes_src,
+						 n->rate_pkts_src, 1);
+
 		printw(" => ");
 	}
 
-	/* Show dest information: reverse DNS, port, country, city */
+	/* Show dest information: reverse DNS, port, country, city, counters */
 	attron(COLOR_PAIR(2));
 	mvwprintw(screen, ++(*line), 8, "dst: %s", n->rev_dns_dst);
 	attroff(COLOR_PAIR(2));
@@ -856,61 +1018,81 @@ static void presenter_screen_do_line(WINDOW *screen, struct flow_entry *n,
 
 		printw(")");
 	}
+
+	if (n->pkts_dst > 0 && n->bytes_dst > 0)
+		presenter_print_counters(n->bytes_dst, n->pkts_dst,
+					 n->rate_bytes_dst,
+					 n->rate_pkts_dst, 2);
 }
 
-static inline int presenter_flow_wrong_state(struct flow_entry *n, int state)
+static inline bool presenter_flow_wrong_state(struct flow_entry *n)
 {
-	int ret = 1;
-
 	switch (n->l4_proto) {
 	case IPPROTO_TCP:
-		if (n->tcp_state == state)
-			ret = 0;
+		switch (n->tcp_state) {
+		case TCP_CONNTRACK_SYN_SENT:
+		case TCP_CONNTRACK_SYN_RECV:
+		case TCP_CONNTRACK_ESTABLISHED:
+		case TCP_CONNTRACK_FIN_WAIT:
+		case TCP_CONNTRACK_CLOSE_WAIT:
+		case TCP_CONNTRACK_LAST_ACK:
+		case TCP_CONNTRACK_TIME_WAIT:
+		case TCP_CONNTRACK_CLOSE:
+		case TCP_CONNTRACK_SYN_SENT2:
+		case TCP_CONNTRACK_NONE:
+			return false;
+			break;
+		}
 		break;
 	case IPPROTO_SCTP:
-		if (n->sctp_state == state)
-			ret = 0;
+		switch (n->sctp_state) {
+		case SCTP_CONNTRACK_NONE:
+		case SCTP_CONNTRACK_CLOSED:
+		case SCTP_CONNTRACK_COOKIE_WAIT:
+		case SCTP_CONNTRACK_COOKIE_ECHOED:
+		case SCTP_CONNTRACK_ESTABLISHED:
+		case SCTP_CONNTRACK_SHUTDOWN_SENT:
+		case SCTP_CONNTRACK_SHUTDOWN_RECD:
+		case SCTP_CONNTRACK_SHUTDOWN_ACK_SENT:
+			return false;
+			break;
+		}
 		break;
 	case IPPROTO_DCCP:
-		if (n->dccp_state == state)
-			ret = 0;
+		switch (n->dccp_state) {
+		case DCCP_CONNTRACK_NONE:
+		case DCCP_CONNTRACK_REQUEST:
+		case DCCP_CONNTRACK_RESPOND:
+		case DCCP_CONNTRACK_PARTOPEN:
+		case DCCP_CONNTRACK_OPEN:
+		case DCCP_CONNTRACK_CLOSEREQ:
+		case DCCP_CONNTRACK_CLOSING:
+		case DCCP_CONNTRACK_TIMEWAIT:
+		case DCCP_CONNTRACK_IGNORE:
+		case DCCP_CONNTRACK_INVALID:
+			return false;
+			break;
+		}
 		break;
 	case IPPROTO_UDP:
 	case IPPROTO_UDPLITE:
 	case IPPROTO_ICMP:
 	case IPPROTO_ICMPV6:
-		ret = 0;
+		return false;
 		break;
 	}
 
-	return ret;
+	return true;
 }
 
 static void presenter_screen_update(WINDOW *screen, struct flow_list *fl,
 				    int skip_lines)
 {
 	int maxy;
-	size_t i, j;
+	int skip_left = skip_lines;
+	unsigned int flows = 0;
 	unsigned int line = 3;
 	struct flow_entry *n;
-	uint8_t protocols[] = {
-		IPPROTO_TCP,
-		IPPROTO_DCCP,
-		IPPROTO_SCTP,
-		IPPROTO_UDP,
-		IPPROTO_UDPLITE,
-		IPPROTO_ICMP,
-		IPPROTO_ICMPV6,
-	};
-	size_t protocol_state_size[] = {
-		[IPPROTO_TCP] = array_size(tcp_states),
-		[IPPROTO_DCCP] = array_size(dccp_states),
-		[IPPROTO_SCTP] = array_size(sctp_states),
-		[IPPROTO_UDP] = 1,
-		[IPPROTO_UDPLITE] = 1,
-		[IPPROTO_ICMP] = 1,
-		[IPPROTO_ICMPV6] = 1,
-	};
 
 	curs_set(0);
 
@@ -926,48 +1108,51 @@ static void presenter_screen_update(WINDOW *screen, struct flow_list *fl,
 	wclear(screen);
 	clear();
 
-	mvwprintw(screen, 1, 2, "Kernel netfilter flows for %s%s%s%s%s%s"
-		  "[+%d]", what & INCLUDE_TCP ? "TCP, " : "" ,
-		  what & INCLUDE_UDP ? "UDP, " : "",
-		  what & INCLUDE_SCTP ? "SCTP, " : "",
-		  what & INCLUDE_DCCP ? "DCCP, " : "",
-		  what & INCLUDE_ICMP && what & INCLUDE_IPV4 ? "ICMP, " : "",
-		  what & INCLUDE_ICMP && what & INCLUDE_IPV6 ? "ICMP6, " : "",
-		  skip_lines);
-
 	rcu_read_lock();
 
-	if (rcu_dereference(fl->head) == NULL)
+	n = rcu_dereference(fl->head);
+	if (!n)
 		mvwprintw(screen, line, 2, "(No active sessions! "
 			  "Is netfilter running?)");
 
-	for (i = 0; i < array_size(protocols); i++) {
-		for (j = 0; j < protocol_state_size[protocols[i]]; j++) {
-			n = rcu_dereference(fl->head);
-			while (n && maxy > 0) {
-				if (n->l4_proto != protocols[i] ||
-				    presenter_flow_wrong_state(n, j) ||
-				    presenter_get_port(n->port_src,
-						       n->port_dst, 0) == 53) {
-					/* skip entry */
-					n = rcu_dereference(n->next);
-					continue;
-				}
-				if (skip_lines > 0) {
-					n = rcu_dereference(n->next);
-					skip_lines--;
-					continue;
-				}
+	for (; n; n = rcu_dereference(n->next)) {
+		n->is_visible = false;
 
-				presenter_screen_do_line(screen, n, &line);
+		if (presenter_flow_wrong_state(n))
+			continue;
 
-				line++;
-				maxy -= (2 + 1 * show_src);
-				n = rcu_dereference(n->next);
-			}
+		/* count only flows which might be showed */
+		flows++;
+
+		if (maxy <= 0)
+			continue;
+
+		if (skip_left > 0) {
+			skip_left--;
+			continue;
 		}
+
+		n->is_visible = true;
+
+		presenter_screen_do_line(screen, n, &line);
+
+		line++;
+		maxy -= (2 + (show_src ? 1 : 0));
 	}
 
+	mvwprintw(screen, 1, 2,
+		"Kernel netfilter flows(%u) for %s%s%s%s%s%s"
+		"[+%d]", flows, what & INCLUDE_TCP ? "TCP, " : "",
+		what & INCLUDE_UDP  ? "UDP, "  : "",
+		what & INCLUDE_SCTP ? "SCTP, " : "",
+		what & INCLUDE_DCCP ? "DCCP, " : "",
+		what & INCLUDE_ICMP && what & INCLUDE_IPV4 ? "ICMP, " : "",
+		what & INCLUDE_ICMP && what & INCLUDE_IPV6 ? "ICMP6, " : "",
+		skip_lines);
+
+	if (is_flow_collecting)
+		printw(" [Collecting flows ...]");
+
 	rcu_read_unlock();
 
 	wrefresh(screen);
@@ -976,17 +1161,19 @@ static void presenter_screen_update(WINDOW *screen, struct flow_list *fl,
 
 static void presenter(void)
 {
+	int time_sleep_us = 200000;
+	int time_passed_us = 0;
 	int skip_lines = 0;
 	WINDOW *screen;
 
-	condlock_wait(&collector_ready);
-
 	lookup_init_ports(PORTS_TCP);
 	lookup_init_ports(PORTS_UDP);
 	screen = screen_init(false);
 
 	rcu_register_thread();
 	while (!sigint) {
+		bool do_redraw = true;
+
 		switch (getch()) {
 		case 'q':
 			sigint = 1;
@@ -1007,11 +1194,21 @@ static void presenter(void)
 			break;
 		default:
 			fflush(stdin);
+			do_redraw = false;
 			break;
 		}
 
-		presenter_screen_update(screen, &flow_list, skip_lines);
-		usleep(100000);
+		if (!do_redraw)
+			do_redraw = time_passed_us >= 1 * USEC_PER_SEC;
+
+		if (do_redraw) {
+			presenter_screen_update(screen, &flow_list, skip_lines);
+			time_passed_us = 0;
+		} else {
+			time_passed_us += time_sleep_us;
+		}
+
+		usleep(time_sleep_us);
 	}
 	rcu_unregister_thread();
 
@@ -1020,8 +1217,8 @@ static void presenter(void)
 	lookup_cleanup_ports(PORTS_TCP);
 }
 
-static int collector_cb(enum nf_conntrack_msg_type type,
-			struct nf_conntrack *ct, void *data __maybe_unused)
+static int flow_event_cb(enum nf_conntrack_msg_type type,
+			 struct nf_conntrack *ct, void *data __maybe_unused)
 {
 	if (sigint)
 		return NFCT_CB_STOP;
@@ -1048,34 +1245,104 @@ static int collector_cb(enum nf_conntrack_msg_type type,
 	return NFCT_CB_CONTINUE;
 }
 
-static inline void collector_flush(struct nfct_handle *handle, uint8_t family)
+static void restore_sysctl(void *obj)
 {
-	nfct_query(handle, NFCT_Q_FLUSH, &family);
+	struct sysctl_params_ctx *sysctl_ctx = obj;
+
+	if (sysctl_ctx->nfct_acct == 0)
+		sysctl_set_int("net/netfilter/nf_conntrack_acct",
+				sysctl_ctx->nfct_acct);
+
+	if (sysctl_ctx->nfct_tstamp == 0)
+		sysctl_set_int("net/netfilter/nf_conntrack_timestamp",
+				sysctl_ctx->nfct_tstamp);
 }
 
-static void *collector(void *null __maybe_unused)
+static void on_panic_handler(void *arg)
+{
+	restore_sysctl(arg);
+	screen_end();
+}
+
+static void conntrack_acct_enable(void)
+{
+	/* We can still work w/o traffic accounting so just warn about error */
+	if (sysctl_get_int("net/netfilter/nf_conntrack_acct", &sysctl.nfct_acct)) {
+		fprintf(stderr, "Can't read net/netfilter/nf_conntrack_acct: %s\n",
+			strerror(errno));
+		return;
+	}
+
+	if (sysctl.nfct_acct == 1)
+		return;
+
+	if (sysctl_set_int("net/netfilter/nf_conntrack_acct", 1)) {
+		fprintf(stderr, "Can't write net/netfilter/nf_conntrack_acct: %s\n",
+			strerror(errno));
+	}
+}
+
+static void conntrack_tstamp_enable(void)
+{
+	if (sysctl_get_int("net/netfilter/nf_conntrack_timestamp", &sysctl.nfct_tstamp)) {
+		fprintf(stderr, "Can't read net/netfilter/nf_conntrack_timestamp: %s\n",
+			strerror(errno));
+		return;
+	}
+
+	if (sysctl.nfct_tstamp == 1)
+		return;
+
+	if (sysctl_set_int("net/netfilter/nf_conntrack_timestamp", 1)) {
+		fprintf(stderr, "Can't write net/netfilter/nf_conntrack_timestamp: %s\n",
+			strerror(errno));
+	}
+}
+
+static int flow_update_cb(enum nf_conntrack_msg_type type,
+			  struct nf_conntrack *ct, void *data __maybe_unused)
+{
+	struct flow_entry *n;
+
+	if (type != NFCT_T_UPDATE)
+		return NFCT_CB_CONTINUE;
+
+	if (sigint)
+		return NFCT_CB_STOP;
+
+	n = flow_list_find_id(&flow_list, nfct_get_attr_u32(ct, ATTR_ID));
+	if (!n)
+		return NFCT_CB_CONTINUE;
+
+	flow_entry_calc_rate(n, ct);
+	flow_entry_update_time(n);
+	flow_entry_from_ct(n, ct);
+
+	return NFCT_CB_CONTINUE;
+}
+
+static void collector_refresh_flows(struct nfct_handle *handle)
+{
+	struct flow_entry *n;
+
+	n = rcu_dereference(flow_list.head);
+	for (; n; n = rcu_dereference(n->next)) {
+		if (!n->is_visible)
+			continue;
+
+		nfct_query(handle, NFCT_Q_GET, n->ct);
+	}
+}
+
+static void collector_create_filter(struct nfct_handle *nfct)
 {
-	int ret;
-	struct nfct_handle *handle;
 	struct nfct_filter *filter;
-
-	handle = nfct_open(CONNTRACK, NF_NETLINK_CONNTRACK_NEW |
-				      NF_NETLINK_CONNTRACK_UPDATE |
-				      NF_NETLINK_CONNTRACK_DESTROY);
-	if (!handle)
-		panic("Cannot create a nfct handle: %s\n", strerror(errno));
-
-	collector_flush(handle, AF_INET);
-	collector_flush(handle, AF_INET6);
+	int ret;
 
 	filter = nfct_filter_create();
 	if (!filter)
 		panic("Cannot create a nfct filter: %s\n", strerror(errno));
 
-	ret = nfct_filter_attach(nfct_fd(handle), filter);
-	if (ret < 0)
-		panic("Cannot attach filter to handle: %s\n", strerror(errno));
-
 	if (what & INCLUDE_UDP) {
 		nfct_filter_add_attr_u32(filter, NFCT_FILTER_L4PROTO, IPPROTO_UDP);
 		nfct_filter_add_attr_u32(filter, NFCT_FILTER_L4PROTO, IPPROTO_UDPLITE);
@@ -1099,25 +1366,175 @@ static void *collector(void *null __maybe_unused)
 		nfct_filter_add_attr(filter, NFCT_FILTER_SRC_IPV6, &filter_ipv6);
 	}
 
-	ret = nfct_filter_attach(nfct_fd(handle), filter);
+	ret = nfct_filter_attach(nfct_fd(nfct), filter);
 	if (ret < 0)
 		panic("Cannot attach filter to handle: %s\n", strerror(errno));
 
-	nfct_callback_register(handle, NFCT_T_ALL, collector_cb, NULL);
 	nfct_filter_destroy(filter);
+}
+
+/* This hand-crafted filter looks ugly but it allows to do not
+ * flush nfct connections & filter them by user specified filter.
+ * May be it is better to replace this one by nfct_cmp. */
+static int flow_dump_cb(enum nf_conntrack_msg_type type,
+			struct nf_conntrack *ct, void *data __maybe_unused)
+{
+	struct flow_entry fl;
+	struct flow_entry *n = &fl;
+
+	if (sigint)
+		return NFCT_CB_STOP;
+
+	synchronize_rcu();
+	spinlock_lock(&flow_list.lock);
+
+	if (!(what & ~(INCLUDE_IPV4 | INCLUDE_IPV6)))
+		goto check_addr;
+
+	CP_NFCT(l4_proto, ATTR_ORIG_L4PROTO, 8);
+
+	if (what & INCLUDE_UDP) {
+		if (n->l4_proto == IPPROTO_UDP)
+			goto check_addr;
+
+		if (n->l4_proto == IPPROTO_UDPLITE)
+			goto check_addr;
+
+	}
+	if ((what & INCLUDE_TCP) && n->l4_proto == IPPROTO_TCP)
+		goto check_addr;
+
+	if ((what & INCLUDE_DCCP) && n->l4_proto == IPPROTO_DCCP)
+		goto check_addr;
+
+	if ((what & INCLUDE_SCTP) && n->l4_proto == IPPROTO_SCTP)
+		goto check_addr;
+
+	if ((what & INCLUDE_ICMP) && (what & INCLUDE_IPV4) &&
+			n->l4_proto == IPPROTO_ICMP) {
+		goto check_addr;
+	}
+
+	if ((what & INCLUDE_ICMP) && (what & INCLUDE_IPV6) &&
+			n->l4_proto == IPPROTO_ICMPV6) {
+		goto check_addr;
+	}
+
+	goto skip_flow;
+
+check_addr:
+	/* filter loopback addresses */
+	if (what & INCLUDE_IPV4) {
+		CP_NFCT(ip4_src_addr, ATTR_ORIG_IPV4_SRC, 32);
+
+		if (n->ip4_src_addr == filter_ipv4.addr)
+			goto skip_flow;
+	}
+	if (what & INCLUDE_IPV6) {
+		CP_NFCT_BUFF(ip6_src_addr, ATTR_ORIG_IPV6_SRC);
+
+		if (n->ip6_src_addr[0] == 0x0 &&
+		    n->ip6_src_addr[1] == 0x0 &&
+		    n->ip6_src_addr[2] == 0x0 &&
+		    n->ip6_src_addr[3] == 0x1)
+			goto skip_flow;
+	}
+
+	flow_list_new_entry(&flow_list, ct);
+
+skip_flow:
+	spinlock_unlock(&flow_list.lock);
+	return NFCT_CB_CONTINUE;
+}
+
+static void collector_dump_flows(void)
+{
+	struct nfct_handle *nfct = nfct_open(CONNTRACK, 0);
+
+	if (!nfct)
+		panic("Cannot create a nfct handle: %s\n", strerror(errno));
+
+	nfct_callback_register(nfct, NFCT_T_ALL, flow_dump_cb, NULL);
+
+	is_flow_collecting = true;
+	if (what & INCLUDE_IPV4) {
+		int family = AF_INET;
+		nfct_query(nfct, NFCT_Q_DUMP, &family);
+	}
+	if (what & INCLUDE_IPV6) {
+		int family = AF_INET6;
+		nfct_query(nfct, NFCT_Q_DUMP, &family);
+	}
+	is_flow_collecting = false;
+
+	nfct_close(nfct);
+}
+
+static void *collector(void *null __maybe_unused)
+{
+	struct nfct_handle *ct_update;
+	struct nfct_handle *ct_event;
+	struct pollfd poll_fd[1];
+
 	flow_list_init(&flow_list);
 
-	condlock_signal(&collector_ready);
+	ct_event = nfct_open(CONNTRACK, NF_NETLINK_CONNTRACK_NEW |
+				      NF_NETLINK_CONNTRACK_UPDATE |
+				      NF_NETLINK_CONNTRACK_DESTROY);
+	if (!ct_event)
+		panic("Cannot create a nfct handle: %s\n", strerror(errno));
+
+	collector_create_filter(ct_event);
+
+	nfct_callback_register(ct_event, NFCT_T_ALL, flow_event_cb, NULL);
+
+	ct_update = nfct_open(CONNTRACK, NF_NETLINK_CONNTRACK_UPDATE);
+	if (!ct_update)
+		panic("Cannot create a nfct handle: %s\n", strerror(errno));
+
+	nfct_callback_register(ct_update, NFCT_T_ALL, flow_update_cb, NULL);
+
+	poll_fd[0].fd = nfct_fd(ct_event);
+	poll_fd[0].events = POLLIN;
+
+	if (fcntl(nfct_fd(ct_event), F_SETFL, O_NONBLOCK) == -1)
+		panic("Cannot set non-blocking socket: fcntl(): %s\n",
+		      strerror(errno));
+
+	if (fcntl(nfct_fd(ct_update), F_SETFL, O_NONBLOCK) == -1)
+		panic("Cannot set non-blocking socket: fcntl(): %s\n",
+		      strerror(errno));
 
 	rcu_register_thread();
 
-	while (!sigint && ret >= 0)
-		ret = nfct_catch(handle);
+	collector_dump_flows();
+
+	while (!sigint) {
+		int status;
+
+		usleep(USEC_PER_SEC * interval);
+
+		collector_refresh_flows(ct_update);
+
+		status = poll(poll_fd, 1, 0);
+		if (status < 0) {
+			if (errno == EAGAIN || errno == EINTR)
+				continue;
+
+			panic("Error while polling: %s\n", strerror(errno));
+		} else if (status == 0) {
+			continue;
+		}
+
+		if (poll_fd[0].revents & POLLIN)
+			nfct_catch(ct_event);
+	}
 
 	rcu_unregister_thread();
 
 	flow_list_destroy(&flow_list);
-	nfct_close(handle);
+	nfct_close(ct_event);
+	nfct_close(ct_update);
 
 	pthread_exit(NULL);
 }
@@ -1155,12 +1572,24 @@ int main(int argc, char **argv)
 			what_cmd |= INCLUDE_SCTP;
 			break;
 		case 's':
-			show_src = 1;
+			show_src = true;
+			break;
+		case 'b':
+			rate_type = RATE_BITS;
 			break;
 		case 'u':
 			update_geoip();
 			die();
 			break;
+		case 't':
+			interval = strtoul(optarg, NULL, 10);
+			break;
+		case 'n':
+			resolve_dns = false;
+			break;
+		case 'G':
+			resolve_geoip = false;
+			break;
 		case 'h':
 			help();
 			break;
@@ -1172,9 +1601,13 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if (what_cmd > 0)
+	if (what_cmd > 0) {
 		what = what_cmd;
 
+		if (!(what & (INCLUDE_IPV4 | INCLUDE_IPV6)))
+			what |= INCLUDE_IPV4 | INCLUDE_IPV6;
+	}
+
 	rcu_init();
 
 	register_signal(SIGINT, signal_handler);
@@ -1182,9 +1615,13 @@ int main(int argc, char **argv)
 	register_signal(SIGTERM, signal_handler);
 	register_signal(SIGHUP, signal_handler);
 
-	init_geoip(1);
+	panic_handler_add(on_panic_handler, &sysctl);
 
-	condlock_init(&collector_ready);
+	conntrack_acct_enable();
+	conntrack_tstamp_enable();
+
+	if (resolve_geoip)
+		init_geoip(1);
 
 	ret = pthread_create(&tid, NULL, collector, NULL);
 	if (ret < 0)
@@ -1192,9 +1629,10 @@ int main(int argc, char **argv)
 
 	presenter();
 
-	condlock_destroy(&collector_ready);
+	if (resolve_geoip)
+		destroy_geoip();
 
-	destroy_geoip();
+	restore_sysctl(&sysctl);
 
 	return 0;
 }
