@@ -93,7 +93,7 @@ static const struct option long_options[] = {
 	{NULL, 0, NULL, 0}
 };
 
-static const char *copyright = "Please report bugs to <bugs@netsniff-ng.org>\n"
+static const char *copyright = "Please report bugs to <netsniff-ng@googlegroups.com>\n"
 	"Copyright (C) 2009-2013 Daniel Borkmann <dborkma@tik.ee.ethz.ch>\n"
 	"Swiss federal institute of technology (ETH Zurich)\n"
 	"Copyright (C) 2013 Tobias Klauser <tklauser@distanz.ch>\n"
@@ -245,7 +245,7 @@ static int stats_proc_net_dev(const char *ifname, struct ifstat *stats)
 		if (strstr(buff, ifname_colon) == NULL)
 			continue;
 
-		if (sscanf(buff, "%*[a-z0-9 .-]:%llu%llu%llu%llu%llu%llu"
+		if (sscanf(buff, "%*[a-z0-9_ .-]:%llu%llu%llu%llu%llu%llu"
 			   "%llu%*u%llu%llu%llu%llu%llu%llu%llu",
 			   &stats->rx_bytes, &stats->rx_packets,
 			   &stats->rx_errors, &stats->rx_drops,
@@ -313,7 +313,6 @@ retry:
 	}
 
 	if (ret == -EINVAL && try == 0) {
-		memset(&drvinf, 0, sizeof(drvinf));
 		if (ethtool_drvinf(ifname, &drvinf) < 0)
 			goto done;
 
@@ -711,7 +710,6 @@ static void screen_header(WINDOW *screen, const char *ifname, int *voff,
 	int link = ethtool_link(ifname);
 	unsigned int cpus = get_number_cpus();
 
-	memset(&drvinf, 0, sizeof(drvinf));
 	ethtool_drvinf(ifname, &drvinf);
 
 	memset(buff, 0, sizeof(buff));
@@ -1127,6 +1125,12 @@ static void screen_update(WINDOW *screen, const char *ifname, const struct ifsta
 	refresh();
 }
 
+static void on_panic_handler(void *arg)
+{
+	screen_end();
+	fprintf(stderr, "Please check <stderr> for error message\n");
+}
+
 static int screen_main(const char *ifname, uint64_t ms_interval,
 		       unsigned int top_cpus, bool suppress_warnings,
 		       bool omit_header __maybe_unused)
@@ -1137,6 +1141,8 @@ static int screen_main(const char *ifname, uint64_t ms_interval,
 
 	stats_screen = screen_init(true);
 
+	panic_handler_add(on_panic_handler, NULL);
+
 	if (((rate > SPEED_1000 && ms_interval <= 1000) ||
 	     (rate = SPEED_1000 && ms_interval <  1000)) &&
 	     !suppress_warnings)
