@@ -1,6 +1,7 @@
 /*
  * netsniff-ng - the packet sniffing beast
  * Copyright 2009, 2010 Daniel Borkmann.
+ * Copyright 2014, 2015 Tobias Klauser.
  * Subject to the GPL, version 2.
  */
 
@@ -47,6 +48,35 @@ union tpacket_uhdr {
 	(((hdr).h2)->member)
 #endif /* HAVE_TPACKET3 */
 
+static inline uint16_t tpacket_uhdr_vlan_tci(union tpacket_uhdr *hdr, bool v3)
+{
+#ifdef HAVE_TPACKET3
+	if (v3)
+		return hdr->h3->hv1.tp_vlan_tci;
+#endif
+	return 0;
+}
+
+static inline uint16_t tpacket_uhdr_vlan_proto(union tpacket_uhdr *hdr, bool v3)
+{
+#if defined(HAVE_TPACKET3) && defined(TP_STATUS_VLAN_TPID_VALID)
+	if (v3)
+		return hdr->h3->hv1.tp_vlan_tpid;
+#endif
+	return 0;
+}
+
+static inline bool tpacket_has_vlan_info(union tpacket_uhdr *hdr)
+{
+	uint32_t valid = TP_STATUS_VLAN_VALID;
+
+#ifdef TP_STATUS_VLAN_TPID_VALID
+	valid |= TP_STATUS_VLAN_TPID_VALID;
+#endif
+
+	return tpacket_uhdr(*hdr, tp_status, true) & valid;
+}
+
 struct frame_map {
 	struct tpacket2_hdr tp_h __aligned_tpacket;
 	struct sockaddr_ll s_ll __align_tpacket(sizeof(struct tpacket2_hdr));
@@ -115,6 +145,14 @@ static inline void ring_verify_layout(struct ring *ring)
 	bug_on((ring->layout.tp_block_size % RUNTIME_PAGE_SIZE) != 0);
 }
 
+static inline void shrink_ring_layout_generic(struct ring *ring)
+{
+	ring->layout.tp_block_nr >>= 1;
+	ring->layout.tp_frame_nr = ring->layout.tp_block_size /
+				   ring->layout.tp_frame_size *
+				   ring->layout.tp_block_nr;
+}
+
 static inline void tpacket_hdr_clone(struct tpacket2_hdr *thdrd,
 				     struct tpacket2_hdr *thdrs)
 {
@@ -167,8 +205,10 @@ static inline int get_sockopt_tpacket(int sock)
 	return val;
 }
 
+extern void setup_ring_layout_generic(int sock, struct ring *ring, size_t size,
+				      bool jumbo_support);
 extern void mmap_ring_generic(int sock, struct ring *ring);
-extern void alloc_ring_frames_generic(struct ring *ring, int num, size_t size);
+extern void alloc_ring_frames_generic(struct ring *ring, size_t num, size_t size);
 extern void bind_ring_generic(int sock, struct ring *ring, int ifindex, bool tx_only);
 
 #endif /* RING_H */
