@@ -1,7 +1,7 @@
 /*
  * netsniff-ng - the packet sniffing beast
  * Copyright 2009, 2010 Daniel Borkmann.
- * Copyright 2014 Tobias Klauser.
+ * Copyright 2014, 2015 Tobias Klauser.
  * Subject to the GPL, version 2.
  */
 
@@ -59,7 +59,7 @@ static inline size_t rx_ring_get_size(struct ring *ring, bool v3)
 	return v3 ? ring->layout3.tp_block_size : ring->layout.tp_frame_size;
 }
 
-static int get_rx_net_stats(int sock, uint64_t *packets, uint64_t *drops, bool v3)
+int get_rx_net_stats(int sock, uint64_t *packets, uint64_t *drops, bool v3)
 {
 	int ret;
 	union {
@@ -91,7 +91,7 @@ static inline void setup_rx_ring_layout_v3(struct ring *ring __maybe_unused)
 {
 }
 
-static inline int rx_ring_get_num(struct ring *ring, bool v3 __maybe_unused)
+static inline size_t rx_ring_get_num(struct ring *ring, bool v3 __maybe_unused)
 {
 	return ring->layout.tp_frame_nr;
 }
@@ -101,7 +101,8 @@ static inline size_t rx_ring_get_size(struct ring *ring, bool v3 __maybe_unused)
 	return ring->layout.tp_frame_size;
 }
 
-static int get_rx_net_stats(int sock, uint64_t *packets, uint64_t *drops, bool v3 __maybe_unused)
+int get_rx_net_stats(int sock, uint64_t *packets, uint64_t *drops,
+		     bool v3 __maybe_unused)
 {
 	int ret;
 	struct tpacket_stats stats;
@@ -141,20 +142,7 @@ void destroy_rx_ring(int sock, struct ring *ring)
 static void setup_rx_ring_layout(int sock, struct ring *ring, size_t size,
 				 bool jumbo_support, bool v3)
 {
-	fmemset(&ring->layout, 0, sizeof(ring->layout));
-
-	ring->layout.tp_block_size = (jumbo_support ?
-				      RUNTIME_PAGE_SIZE << 4 :
-				      RUNTIME_PAGE_SIZE << 2);
-
-	ring->layout.tp_frame_size = (jumbo_support ?
-				      TPACKET_ALIGNMENT << 12 :
-				      TPACKET_ALIGNMENT << 7);
-
-	ring->layout.tp_block_nr = size / ring->layout.tp_block_size;
-	ring->layout.tp_frame_nr = ring->layout.tp_block_size /
-				   ring->layout.tp_frame_size *
-				   ring->layout.tp_block_nr;
+	setup_ring_layout_generic(sock, ring, size, jumbo_support);
 
 	if (v3) {
 		setup_rx_ring_layout_v3(ring);
@@ -177,10 +165,7 @@ retry:
 			 layout_size);
 
 	if (errno == ENOMEM && ring->layout.tp_block_nr > 1) {
-		ring->layout.tp_block_nr >>= 1;
-		ring->layout.tp_frame_nr = ring->layout.tp_block_size / 
-					   ring->layout.tp_frame_size * 
-					   ring->layout.tp_block_nr;
+		shrink_ring_layout_generic(ring);
 		goto retry;
 	}
 	if (ret < 0)
@@ -238,21 +223,3 @@ void ring_rx_setup(struct ring *ring, int sock, size_t size, int ifindex,
 	join_fanout_group(sock, fanout_group, fanout_type);
 	prepare_polling(sock, poll);
 }
-
-void sock_rx_net_stats(int sock, unsigned long seen)
-{
-	int ret;
-	uint64_t packets, drops;
-	bool v3 = is_tpacket_v3(sock);
-
-	ret = get_rx_net_stats(sock, &packets, &drops, v3);
-	if (ret == 0) {
-		printf("\r%12"PRIu64"  packets incoming (%"PRIu64" unread on exit)\n",
-		       v3 ? (uint64_t)seen : packets, v3 ? packets - seen : 0);
-		printf("\r%12"PRIu64"  packets passed filter\n", packets - drops);
-		printf("\r%12"PRIu64"  packets failed filter (out of space)\n", drops);
-		if (packets > 0)
-			printf("\r%12.4lf%% packet droprate\n",
-			       (1.0 * drops / packets) * 100.0);
-	}
-}
