@@ -8,7 +8,9 @@
 #include "sock.h"
 #include "die.h"
 #include "str.h"
+#include "linktype.h"
 #include "built_in.h"
+#include "sysctl.h"
 
 int af_socket(int af)
 {
@@ -33,6 +35,26 @@ int pf_socket(void)
 	return sock;
 }
 
+static int pf_socket_dgram(void)
+{
+	int sock = socket(PF_PACKET, SOCK_DGRAM, 0);
+	if (unlikely(sock < 0))
+		panic("Creation of PF dgram socket failed: %s\n",
+		      strerror(errno));
+
+	return sock;
+}
+
+int pf_socket_type(uint32_t type)
+{
+	switch (type) {
+	case LINKTYPE_LINUX_SLL:
+		return pf_socket_dgram();
+	default:
+		return pf_socket();
+	}
+}
+
 /* Avail in kernel >= 3.14
  * in commit d346a3fae3 (packet: introduce PACKET_QDISC_BYPASS socket option)
  */
@@ -128,50 +150,27 @@ enum {
 #define SMEM_SUG_DEF	4194304
 
 static const char *const sock_mem[] = {
-	[sock_rmem_max] = "/proc/sys/net/core/rmem_max",
-	[sock_rmem_def] = "/proc/sys/net/core/rmem_default",
-	[sock_wmem_max] = "/proc/sys/net/core/wmem_max",
-	[sock_wmem_def] = "/proc/sys/net/core/wmem_default",
+	[sock_rmem_max] = "net/core/rmem_max",
+	[sock_rmem_def] = "net/core/rmem_default",
+	[sock_wmem_max] = "net/core/wmem_max",
+	[sock_wmem_def] = "net/core/wmem_default",
 };
 
 static int get_system_socket_mem(int which)
 {
-	int fd, val = -1;
-	ssize_t ret;
-	const char *file = sock_mem[which];
-	char buff[64];
+	int val;
 
-	fd = open(file, O_RDONLY);
-	if (fd < 0)
-		return val;
+	if (sysctl_get_int(sock_mem[which], &val))
+		return -1;
 
-	ret = read(fd, buff, sizeof(buff));
-	if (ret > 0)
-		val = atoi(buff);
-
-	close(fd);
 	return val;
 }
 
 static void set_system_socket_mem(int which, int val)
 {
-	int fd;
-	const char *file = sock_mem[which];
-	ssize_t ret;
-	char buff[64];
-
-	fd = open(file, O_WRONLY);
-	if (fd < 0)
-		return;
-
-	memset(buff, 0, sizeof(buff));
-	slprintf(buff, sizeof(buff), "%d", val);
-
-	ret = write(fd, buff, strlen(buff));
-	if (ret < 0)
-		panic("Cannot set system socket memory!\n");
-
-	close(fd);
+	if (val > 0 && sysctl_set_int(sock_mem[which], val))
+		printf("Cannot set system socket memory in %s%s: %s\n",
+		       SYSCTL_PROC_PATH, sock_mem[which], strerror(errno));
 }
 
 void set_system_socket_memory(int *vals, size_t len)
