@@ -113,7 +113,7 @@ static const struct option long_options[] = {
 	{NULL, 0, NULL, 0}
 };
 
-static const char *copyright = "Please report bugs to <bugs@netsniff-ng.org>\n"
+static const char *copyright = "Please report bugs to <netsniff-ng@googlegroups.com>\n"
 	"Copyright (C) 2011-2013 Daniel Borkmann <dborkma@tik.ee.ethz.ch>,\n"
 	"Swiss federal institute of technology (ETH Zurich)\n"
 	"License: GNU GPL version 2.0\n"
@@ -138,6 +138,8 @@ struct icmp_filter {
 
 #define SMOKE_N_PROBES	100
 
+#define PKT_MIN_LEN 14
+
 static void signal_handler(int number)
 {
 	switch (number) {
@@ -343,6 +345,18 @@ static void apply_csum16(int id)
 				      (packets[id].len - csum->to),
 				      IPPROTO_TCP);
 			break;
+		case CSUM_UDP6:
+			sum = p6_csum((void *) packets[id].payload + csum->from,
+				      packets[id].payload + csum->to,
+				      (packets[id].len - csum->to),
+				      IPPROTO_UDP);
+			break;
+		case CSUM_TCP6:
+			sum = p6_csum((void *) packets[id].payload + csum->from,
+				      packets[id].payload + csum->to,
+				      (packets[id].len - csum->to),
+				      IPPROTO_TCP);
+			break;
 		default:
 			bug();
 			break;
@@ -352,6 +366,21 @@ static void apply_csum16(int id)
 	}
 }
 
+static void preprocess_packets(void)
+{
+	size_t i;
+
+	for (i = 0; i < plen; i++) {
+		struct packet_dyn *pktd = &packet_dyn[i];
+
+		if (packet_dyn_has_only_csums(pktd)) {
+			apply_csum16(i);
+			pktd->slen = 0;
+			xfree(pktd->csum);
+		}
+	}
+}
+
 static struct cpu_stats *setup_shared_var(unsigned int cpus)
 {
 	int fd;
@@ -427,7 +456,7 @@ static int xmit_smoke_setup(struct ctx *ctx)
 
 	ret = inet_aton(ctx->rhost, &ctx->dest.sin_addr);
 	if (ret < 0)
-		panic("Cannot resolv address!\n");
+		panic("Cannot resolve address!\n");
 
 	return icmp_sock;
 }
@@ -525,11 +554,13 @@ static void xmit_slowpath_or_die(struct ctx *ctx, unsigned int cpu, unsigned lon
 
 	drop_privileges(ctx->enforce, ctx->uid, ctx->gid);
 
+	preprocess_packets();
+
 	bug_on(gettimeofday(&start, NULL));
 
 	while (likely(sigint == 0 && num > 0 && plen > 0)) {
 		pktd = &packet_dyn[i];
-		if (pktd->clen + pktd->rlen + pktd->slen) {
+		if (packet_dyn_has_elems(pktd)) {
 			apply_counter(i);
 			apply_randomizer(i);
 			apply_csum16(i);
@@ -614,6 +645,8 @@ static void xmit_fastpath_or_die(struct ctx *ctx, unsigned int cpu, unsigned lon
 	if (ctx->num == 0 && orig_num > 0)
 		num = 0;
 
+	preprocess_packets();
+
 	bug_on(gettimeofday(&start, NULL));
 
 	while (likely(sigint == 0 && num > 0 && plen > 0)) {
@@ -637,7 +670,7 @@ static void xmit_fastpath_or_die(struct ctx *ctx, unsigned int cpu, unsigned lon
 		hdr->tp_h.tp_len = packets[i].len;
 
 		pktd = &packet_dyn[i];
-		if (pktd->clen + pktd->rlen + pktd->slen) {
+		if (packet_dyn_has_elems(pktd)) {
 			apply_counter(i);
 			apply_randomizer(i);
 			apply_csum16(i);
@@ -763,11 +796,11 @@ static void __set_state_cd(unsigned int cpu, unsigned long p, sig_atomic_t s)
 	stats[cpu].state = s;
 }
 
-static int xmit_packet_precheck(struct ctx *ctx, unsigned int cpu)
+static void xmit_packet_precheck(struct ctx *ctx, unsigned int cpu)
 {
-	unsigned int i;
 	unsigned long plen_total, orig = ctx->num;
-	size_t mtu, total_len = 0;
+	size_t total_len = 0;
+	unsigned int i;
 
 	bug_on(plen != dlen);
 
@@ -787,25 +820,15 @@ static int xmit_packet_precheck(struct ctx *ctx, unsigned int cpu)
 
 	if (plen == 0) {
 		__set_state(cpu, CPU_STATS_STATE_RES);
-		return 0;
+		return;
 	}
-
-	for (mtu = device_mtu(ctx->device), i = 0; i < plen; ++i) {
-		if (packets[i].len > mtu + 14)
-			panic("Device MTU < than packet%d's size!\n", i);
-		if (packets[i].len <= 14)
-			panic("Packet%d's size too short!\n", i);
-	}
-
-	return 0;
 }
 
 static void main_loop(struct ctx *ctx, char *confname, bool slow,
 		      unsigned int cpu, bool invoke_cpp, unsigned long orig_num)
 {
 	compile_packets(confname, ctx->verbose, cpu, invoke_cpp);
-	if (xmit_packet_precheck(ctx, cpu) < 0)
-		return;
+	xmit_packet_precheck(ctx, cpu);
 
 	if (cpu == 0) {
 		unsigned int i;
